import getpass
import json
import os

import geopandas as gpd
from loguru import logger
from satio_pc.extension import SatioTimeSeries  # noqa
from smart_open import open

from veg_workflows.aws import BaseConfig, get_key_from_s3_uri, get_s3_api

CDSE_ENDPOINT_URL = "https://eodata.dataspace.copernicus.eu/"
VEGAWS_BUCKET = "vito-lcfm"
SENTHUB_BUCKET_NAME = "sentinel-s2-l2a"
AWS_L2A_BUCKET_NAME = SENTHUB_BUCKET_NAME

VEGAWS_OUTPUT_BUCKETNAME = "vito-lcfm"
AWS_LCFM_OUTPUT_BUCKET = VEGAWS_OUTPUT_BUCKETNAME

USERNAME = getpass.getuser()
CDSE_BUCKET_NAME = "EODATA"
COLLECTION_S3_URI = "s3://vito-lcfm/global_processing/collections/lcfm-10_collection_baseV2_senthub_2020-01-01_2020-12-31.csv"
RIO_CONFIG_URI = "s3://vito-lcfm/global_processing/config/rio_config.json"
FGB_10_S3_URI = (
    "s3://vito-lcfm/global_processing/collections/LCFM_10p_S2-tiles.fgb"
)
FGB_100_S3_URI = (
    "s3://vito-lcfm/global_processing/collections/LCFM_100p_S2-tiles.fgb"
)
MODEL_INDEXES = "s3://vito-lcfm/global_processing/models/model_indexes.json"


band_names = {
    60: [
        "B01",
        "B02",
        "B03",
        "B04",
        "B05",
        "B06",
        "B07",
        "B8A",
        "B09",
        "B11",
        "B12",
    ],
    20: ["SCL"],
}


class LoiConfigV2(BaseConfig):
    config_fields = (
        "app_id",
        "collection_uri",
        "model_name",
        "model_idx",
        "version_export",
        "skip_done",
        "output_bucket",
    )

    def __repr__(self):
        return f"LoiConfig({self.app_id})"

    def __str__(self):
        return f"LoiConfig({self.app_id})"


class LOIConfig:
    """Creates a config object for cloud model inference"""

    def __init__(self, config_path: str):
        self.config_path = config_path
        args = self.__load_config(config_path)

        self.model_name = args["model_name"]
        self.model_idx = self.__get_model_index(self.model_name)
        self.version_export = args["version_export"]
        self.collection_uri = args["collection_uri"]
        self.overwrite = args.get("overwrite", False)
        self.skip_done = args.get("skip_done", True)
        self.testing = args.get("testing", False)
        self.local = args.get("local", False)
        self.spark_slices_factor = args.get("spark_slices_factor", 1)
        self.rio_config_path = args.get("rio_config_path", RIO_CONFIG_URI)

    def __repr__(self):
        # Create a dictionary excluding self.s3_vegaws
        repr_dict = {
            k: v for k, v in self.__dict__.items() if k != "s3_vegaws"
        }
        return json.dumps(repr_dict, indent=4)

    @staticmethod
    def __load_config(_config_path: str):
        s3_vegaws_api = get_s3_api(bucket=VEGAWS_BUCKET)
        if _config_path.startswith("s3://"):
            conf_json = get_key_from_s3_uri(_config_path)
            args: dict = s3_vegaws_api.read_json(conf_json)
        else:
            conf_json = _config_path
            with open(conf_json, "r") as f:
                args = json.load(f)
        logger.info(f"Reading configuration from {conf_json}")
        return args

    @staticmethod
    def __get_model_index(model_name):
        s3_vegaws_api = get_s3_api(bucket=VEGAWS_BUCKET)
        model_idx = s3_vegaws_api.read_json(
            get_key_from_s3_uri(MODEL_INDEXES)
        )["model_name"][model_name]
        logger.info(f"Model name: {model_name}, Model index: {model_idx}")
        return model_idx


def get_shapefiles_tilelist(fgb_path):
    shapefiles_path = os.path.join(os.path.dirname(__file__), "shapefiles")
    gdf100 = gpd.read_file(f"{shapefiles_path}/LCFM_100p_S2-tiles.fgb")
    return gdf100["tile"].tolist()


class UTMTile:
    def __init__(self, epsg, tile, bounds, geometry):
        self.epsg = epsg
        self.tile = tile
        self.bounds = bounds
        self.geometry = geometry

    def get_geometry(self):
        return self.geometry

    @classmethod
    def from_geodataframe(cls, tile_value, fgb_path: str):
        with open(fgb_path, "r") as f:
            fgb_file = f.read()
        gdf = gpd.read_file(fgb_file)
        row = gdf[gdf["tile"] == tile_value].iloc[0]
        epsg = row["epsg"]
        tile = row["tile"]
        bounds = eval(
            row["bounds"]
        )  # Assuming 'bounds' is a string representation of a bounding box
        geometry = row["geometry"]
        return cls(epsg, tile, bounds, geometry)


def pipeline_wrapper(product_id_config_tup):
    from veg_workflows.cloud_mask.inference_pipeline import (
        AWSCloudInferencePipeline,
    )

    product_id, config = product_id_config_tup
    cloud_pipeline = AWSCloudInferencePipeline(
        product_id,
        model_name=config.model_name,
        model_idx=config.model_idx,
        version=config.version_export,
        s2_db_uri=config.collection_uri,
        output_bucket=config.output_bucket,
    )
    cloud_pipeline.run(fill_value=10_000, export=True, clean_up=True)
